using System;
using System.Linq;
using PkmnLibSharp.Battling.ChoiceTurn;
using PkmnLibSharp.Battling.Events;
using PkmnLibSharp.Utilities;

namespace PkmnLibSharp.Battling
{
    public class Battle : PointerWrapper
    {
        public Battle(BattleLibrary library, BattleParty[] parties, bool canFlee, byte numberOfSides, byte pokemonPerSide,
            ulong randomSeed)
        {
            var ptr = IntPtr.Zero;
            var arr = parties.Select(x => x.Ptr).ToArray();
            Pkmnlib.Generated.Battle.Construct(ref ptr, library.Ptr, arr.ArrayPtr(), (ulong) arr.Length,
                canFlee.ToNative(), numberOfSides, pokemonPerSide, randomSeed);
            Initialize(ptr);
        }

        public BattleLibrary Library
        {
            get
            {
                if (_library != null) return _library;
                var ptr = Creaturelibbattling.Generated.Battle.GetLibrary(Ptr);
                if (TryResolvePointer(ptr, out _library))
                {
                    return _library;
                }
                _library = new BattleLibrary(ptr);
                return _library;
            }
        }
        
        public BattleRandom Random
        {
            get
            {
                if (_random != null) return _random;
                var ptr = Creaturelibbattling.Generated.Battle.GetRandom(Ptr);
                if (TryResolvePointer(ptr, out _random))
                {
                    return _random;
                }
                _random = new BattleRandom(ptr);
                return _random;
            }
        }

        public bool CanFlee => Creaturelibbattling.Generated.Battle.CanFlee(Ptr) == 1;
        public bool HasEnded => Creaturelibbattling.Generated.Battle.HasEnded(Ptr) == 1;
        public bool HasConclusiveResult => Creaturelibbattling.Generated.Battle.HasConclusiveResult(Ptr) == 1;
        public byte WinningSide => Creaturelibbattling.Generated.Battle.GetWinningSide(Ptr);
        public ulong SidesCount => Creaturelibbattling.Generated.Battle.GetSidesCount(Ptr);
        public ulong PartiesCount => Creaturelibbattling.Generated.Battle.GetPartiesCount(Ptr);

        public ReadOnlyNativePtrArray<BattleSide> Sides
        {
            get
            {
                if (_sides != null) return _sides;
                var ptr = Creaturelibbattling.Generated.Battle.GetSides(Ptr);
                _sides = new ReadOnlyNativePtrArray<BattleSide>(ptr, (int) SidesCount);
                return _sides;
            }
        }
        
        public ReadOnlyNativePtrArray<BattleParty> Parties
        {
            get
            {
                if (_parties != null) return _parties;
                var ptr = Creaturelibbattling.Generated.Battle.GetParties(Ptr);
                _parties = new ReadOnlyNativePtrArray<BattleParty>(ptr, (int) PartiesCount);
                return _parties;
            }
        }
        
        public string WeatherName => Pkmnlib.Generated.Battle.GetWeatherName(Ptr).PtrString();


        public bool CanUse(BaseTurnChoice turnChoice)
        {
            byte b = 0;
            Creaturelibbattling.Generated.Battle.CanUse(ref b, Ptr, turnChoice.Ptr).Assert();
            return b == 1;
        }

        public bool TrySetChoice(BaseTurnChoice turnChoice)
        {
            byte b = 0;
            Creaturelibbattling.Generated.Battle.TrySetChoice(ref b, Ptr, turnChoice.Ptr).Assert();
            return b == 1;
        }

        public void CheckChoicesSetAndRun()
        {
            Creaturelibbattling.Generated.Battle.CheckChoicesSetAndRun(Ptr);
        }

        public bool IsPokemonInField(Pokemon pokemon)
        {
            byte b = 0;
            Creaturelibbattling.Generated.Battle.CreatureInField(ref b, Ptr, pokemon.Ptr).Assert();
            return b == 1;
        }

        public Pokemon GetPokemonInField(byte side, byte index)
        {
            var ptr = IntPtr.Zero;
            Creaturelibbattling.Generated.Battle.GetCreature(ref ptr, Ptr, side, index).Assert();
            return TryResolvePointer(ptr, out Pokemon pokemon) ? pokemon : new Pokemon(ptr);
        }

        public void ForceRecall(byte side, byte index)
        {
            Creaturelibbattling.Generated.Battle.ForceRecall(Ptr, side, index).Assert();
        }
        
        public void SwitchPokemon(byte side, byte index, Pokemon newPokemon)
        {
            Creaturelibbattling.Generated.Battle.SwitchCreature(Ptr, side, index, newPokemon.Ptr).Assert();
        }

        public bool CanSlotBeFilled(byte side, byte index)
        {
            byte b = 0;
            Creaturelibbattling.Generated.Battle.CanSlotBeFilled(ref b, Ptr, side, index).Assert();
            return b == 1;
        }

        public void ValidateBattleState()
        {
            Creaturelibbattling.Generated.Battle.ValidateBattleState(Ptr).Assert();
        }
        
        public Script GetVolatileScript(string key)
        {
            var ptr = Creaturelibbattling.Generated.Battle.GetVolatileScript(Ptr, key.ToPtr());
            if (TryResolvePointer(ptr, out Script script))
            {
                return script;
            }
            // TODO: Handle with different script providers.
            return new AngelscriptScript(ptr);
        }

        public void AddVolatileScript(string key)
        {
            Creaturelibbattling.Generated.Battle.AddVolatileScriptByName(Ptr, key.ToPtr()).Assert();
        }
        
        public void AddVolatileScript(Script script)
        {
            Creaturelibbattling.Generated.Battle.AddVolatileScript(Ptr, script.Ptr).Assert();
        }
        
        public void RemoveVolatileScript(string key)
        {
            Creaturelibbattling.Generated.Battle.RemoveVolatileScript(Ptr, key.ToPtr()).Assert();
        }
        
        public void RemoveVolatileScript(Script script)
        {
            Creaturelibbattling.Generated.Battle.RemoveVolatileScriptWithScript(Ptr, script.Ptr).Assert();
        }
        
        public bool HasVolatileScript(string key)
        {
            return Creaturelibbattling.Generated.Battle.HasVolatileScript(Ptr, key.ToPtr()) == 1;
        }

        public void RegisterEventListener(BattleEventListener listener)
        {
            Creaturelibbattling.Generated.Battle.RegisterEventListener(Ptr, listener.FunctionPtr).Assert();
        }

        public void TriggerEventListener(BattleEvent evt)
        {
            Creaturelibbattling.Generated.Battle.TriggerEventListener(Ptr, evt.Ptr).Assert();
        }

        public void SetWeather(string weatherName)
        {
            Pkmnlib.Generated.Battle.SetWeather(Ptr, weatherName.ToPtr());
        }
        
        public void ClearWeather()
        {
            Pkmnlib.Generated.Battle.ClearWeather(Ptr);
        }

        private BattleLibrary _library;
        private BattleRandom _random;
        private ReadOnlyNativePtrArray<BattleSide> _sides;
        private ReadOnlyNativePtrArray<BattleParty> _parties;
        
        protected override void DeletePtr()
        {
            Pkmnlib.Generated.Battle.Destruct(Ptr);
        }
    }
}